from ..utils import minimum_version


class SystemApiMixin:
    def system_status_get(self, **kwargs):
        """
        This method allows you to query NETLAB+ system information.

        :return: Returns a system status object with the following properties:
        :rtype: dict

        **Properties**

        cpu_n
            Number of CPUs detected by the virtual appliance.
        uptime_sec
            System uptime in seconds.
        hostname
            NETLAB+ domain name
        sys_lic_exp_date
            License expiration date
        sys_lic_op_state
            License operation stsate
        sys_logins_enabled
            User logins status
        sys_maint_ends
            Mainenance end date
        sys_mode
            System mode
        sys_name
            System name
        sys_product_id
            Product identifier
        sys_sdn_release_date
            Software distribution date
        sys_sdn_release_type
            Software distribution release type
        sys_sdn_version
            Software distribution version
        sys_serial
            System serial number
        """
        method = "system.status.get"
        return self.call(method, **kwargs)

    @minimum_version('17.1.3')
    def system_usage_cpu(self, **kwargs):
        """
        This method allows you to query NETLAB+ CPU usage.

        :return: Returns an object with N+1 properties, where N is the number of CPUs. The CPUs are listed from 0 to
                 N-1 and also includes **all** . They each contain an object with the following properties:
        :rtype: dict

        **Properties**

        gnice_pct
            The percentage of time spent by the CPU or CPUs to run a niced guest.
        guest_pct
            The percentage of time spent by the CPU or CPUs to run a virtual processor.
        idle_pct
            The percentage of time that the CPU or CPUs were idle and the system did not have an outstanding disk I/O
            request.
        iowait_pct
            The percentage of time that the CPU or CPUs were idle during which the system had an outstanding disk I/O
            request.
        irq_pct
            The percentage of time spent by the CPU or CPUs to service hardware interrupts.
        nice_pct
            The percentage of CPU utilization that occurred while executing at the user level with nice priority.
        soft_pct
            The percentage of time spent by the CPU or CPUs to service software interrupts.
        steal_pct
            The percentage of time spent in involuntary wait by the virtual CPU or CPUs while the hypervisor was
            servicing another virtual processor.
        sys_pct
            The percentage of CPU utilization that occurred while executing at the system level (kernel). Note that
            this does not include time spent servicing hardware and software interrupts.
        usr_pct
            The percentage of CPU utilization that occurred while executing at the user level (application).
        """
        method = "system.usage.cpu"
        return self.call(method, **kwargs)

    @minimum_version('17.1.3')
    def system_usage_disk(self, **kwargs):
        """
        This method allows you to query NETLAB+ disk usage.

        :return: Returns an object with two properties, **data** and **programs** . They each contain an object with the
                 following properties:
        :rtype: dict

        **Properties**

        avail_b
            Available disk space in bytes.
        total_b
            Total disk space in bytes.
        used_b
            Used disk space in bytes.
        used_pct
            Disk space used as a percentage of the whole.
        """
        method = "system.usage.disk"
        return self.call(method, **kwargs)

    @minimum_version('17.1.3')
    def system_usage_memory(self, **kwargs):
        """
        This method allows you to query NETLAB+ memory usage.

        :return: Returns an object with a single property, **ram**. It contains an object with the following properties:
        :rtype: dict

        **Properties**

        free_b
            Free memory in bytes.
        total_b
            Total memory in bytes.
        used_b
            Used memory in bytes.
        """
        method = "system.usage.memory"
        return self.call(method, **kwargs)

    @minimum_version('17.1.4')
    def system_time_timezone_get(self, *, tz_id, **kwargs):
        """
        Match a netlab timezone id to the timezone iana.

        :return: Returns an object that represents a timezone.
        :rtype: dict

        **Properties**

        tz_id
            The id of timezone (specific to netlab).
        tz_iana
            The iana identifier of the timezone.
        tz_name
            The name of the timezone.
        tz_sort
            The sort order of the timezone.
        """
        method = 'system.time.timezone.get'
        return self.call(method, tz_id=tz_id, **kwargs)

    @minimum_version('17.1.4')
    def system_time_timezone_list(self, **kwargs):
        """
        Match a netlab timezone id to the timezone iana.

        :return: Returns an object that represents a timezone.
        :rtype: dict

        **Properties**

        tz_id
            The id of timezone (specific to netlab).
        tz_iana
            The iana identifier of the timezone.
        tz_name
            The name of the timezone.
        tz_sort
            The sort order of the timezone.
        """

        method = 'system.time.timezone.list'
        return self.call(method, **kwargs)



from .common import NetlabError


CODES = []


def _add(cls):
    CODES.append(cls)
    return cls


@_add
class CommunityNotFoundError(NetlabError):
    """A pod could not be located from that request."""
    str_error = "E_COMMUNITY_NOT_FOUND"


@_add
class NameNotUniqueError(NetlabError):
    """acc_user_id already exists in the community."""
    str_error = "E_NAME_NOT_UNIQUE"


@_add
class ClassCommunityMismatchError(NetlabError):
    """cls_id exists but is not in com_id."""
    str_error = "E_CLASS_COMMUNITY_MISMATCH"


@_add
class ClassNotLeadError(NetlabError):
    """Requester acc_id must be a class lead."""
    str_error = "E_CLASS_NOT_LEAD"


@_add
class DatabaseInsertFailedError(NetlabError):
    """Unable to insert record."""
    str_error = "E_DB_INSERT_FAILED"


@_add
class PasswordRequiredError(NetlabError):
    """new_password is required."""
    str_error = "E_PASSWORD_REQUIRED"


@_add
class PasswordTooShortError(NetlabError):
    """new_password is < 7."""
    str_error = "E_PASSWORD_TOO_SHORT"


@_add
class PasswordTooLongError(NetlabError):
    """new_password is > 64."""
    str_error = "E_PASSWORD_TOO_LONG"


@_add
class PasswordLeadingWhitespaceError(NetlabError):
    """There is whitespace before new_password."""
    str_error = "E_PASSWORD_LEADING_WHITESPACE"


@_add
class PasswordTrailingWhitespaceError(NetlabError):
    """There is whitespace after new_password."""
    str_error = "E_PASSWORD_TRAILING_WHITESPACE"


@_add
class PasswordUniqueCharsError(NetlabError):
    """new_password requires at least 5 unique characters."""
    str_error = "E_PASSWORD_UNIQUE_CHARS"


@_add
class PasswordIsCurrentError(NetlabError):
    """new_password is the same as current password."""
    str_error = "E_PASSWORD_IS_CURRENT"


@_add
class PasswordIsUserIdError(NetlabError):
    """new_password is the same as user id."""
    str_error = "E_PASSWORD_IS_USERID"


@_add
class PasswordIsFamilyNameError(NetlabError):
    """new_password is the same as account family name."""
    str_error = "E_PASSWORD_IS_FAMILY_NAME"


@_add
class PasswordIsGivenNameError(NetlabError):
    """new_password is the same as account given name."""
    str_error = "E_PASSWORD_IS_GIVEN_NAME"


@_add
class PasswordIsFullNameError(NetlabError):
    """new_password is the same as account full name."""
    str_error = "E_PASSWORD_IS_FULL_NAME"


@_add
class PasswordIsEmailError(NetlabError):
    """new_password is the same as account email."""
    str_error = "E_PASSWORD_IS_EMAIL"


@_add
class DictionaryError(NetlabError):
    """Could not access the common word dictionary."""
    str_error = "E_DICTIONARY_ERROR"


@_add
class PasswordCommonError(NetlabError):
    """new_password is a common word."""
    str_error = "E_PASSWORD_COMMON"


@_add
class AccountNotFoundError(NetlabError):
    """acc_id not found."""
    str_error = "E_ACCOUNT_NOT_FOUND"


@_add
class UserIdRequiredError(NetlabError):
    """user_id is required."""
    str_error = "E_USER_ID_REQUIRED"


@_add
class UserIdReserveredError(NetlabError):
    """The following names are reserved for NDG and NETLAB+, administrator system techsupport"""
    str_error = "E_USER_ID_RESERVED"


@_add
class UserIdNotUniqueError(NetlabError):
    """This name is already in use."""
    str_error = "E_USER_ID_NOT_UNIQUE"


@_add
class InvalidUserIdError(NetlabError):
    """user_id is invalid."""
    str_error = "E_INVALID_USER_ID"


@_add
class LoginsDisabledSystemError(NetlabError):
    """Logins have been disabled globally on system."""
    str_error = "E_LOGINS_DISABLED_SYSTEM"


@_add
class SystemTerminatedError(NetlabError):
    """System was terminated by NDG."""
    str_error = "E_SYSTEM_TERMINATED"


@_add
class InvalidSessionError(NetlabError):
    """acc_session_id is invalid."""
    str_error = "E_INVALID_SESSION"


@_add
class InvalidTransactionError(NetlabError):
    """TX number invalid."""
    str_error = "E_INVALID_TRANSACTION"



from enum import Enum


class ReservationType(Enum):
    ""
    INDIVIDUAL = 'S'  # 'INDIVIDUAL'
    "Individual reservation type."
    ILT_CLASS = 'C'  # 'ILT_CLASS'
    "ILT class"
    TEAM = 'T'  # 'TEAM'
    "Team"
    INSTRUCTOR = 'I'  # 'INSTRUCTOR'
    "Instructor"


class ReservationScope(Enum):
    "Reservation scope."
    RELEVANT = 'RELEVANT'
    "Relevant to caller, i.e. reservations associated with you in context to user role."
    ALL = 'ALL'
    "All reservations."
    COMMUNITY = 'COMMUNITY'
    "Relevant to the community."
    CLASS = 'CLASS'
    "Relevant to your class."


class InitConfig(Enum):
    "Initial configuration."
    NONE = 'N'
    "No configs loaded."
    LAST = 'L'
    "Load last saved config (if any)."
    EXERCISE = 'E'
    "Load default for exercise (if any)."



from datetime import datetime, timedelta

from netlab.client import Client
from netlab.errors.common import NetlabError
from netlab.errors.user import AccountNotFoundError

#####################
# Variables to change
#####################

POD_ID = 4402  # The pod to reserve
CLASS_ID = 2  # The class this reservation is for. Can only be set to None if the type below is INSTRUCTOR
RESERVATION_TYPE = 'ILT_CLASS'  # choose from ('INDIVIDUAL'|'ILT_CLASS'|'TEAM'|'INSTRUCTOR')
RESERVATION_USERNAME = ''  # This is the user you want to make the reservation for.

RESERVATION_TIME_ZONE = 'America/Chicago'  # Find your IANA timezone here - https://ndg.tech/hReahU
RESERVATION_START_TIME = '09:00'  # in HH:MM format, 24 hour time
RESERVATION_END_TIME = '12:00'  # in HH:MM format, 24 hour time
RESERVATION_FIRST_DAY = '2017-01-23'  # in YYYY-MM-DD format
RESERVATION_LAST_DAY = '2017-05-12'  # in YYYY-MM-DD format
# days of the week the reservations should be created, separated by spaces: M T W Th F Sa Su
RESERVATION_REPEAT_DAYS = 'M W F'  # This will repeat on Monday, Wednesday and Friday

NETLAB_SYSTEM = 'DEMO'  # must be a valid entry in your config.json

#########################
# Don't modify code below
#########################

days_of_the_week = ['M', 'T', 'W', 'Th', 'F', 'Sa', 'Su']
repeat_days = RESERVATION_REPEAT_DAYS.split(" ")

client = Client(NETLAB_SYSTEM)
reservations_created = 0
reservations_with_errors = 0

# find acc_id by looking up the username
results = client.user_account_search(properties='acc_id', filter={'acc_user_id': RESERVATION_USERNAME})
if results:
    acc_id = results[0]['acc_id']
else:
    raise AccountNotFoundError(RESERVATION_USERNAME)

# turn date strings into datetime objects
first_day = datetime.strptime("{}".format(RESERVATION_FIRST_DAY), '%Y-%m-%d')
last_day = datetime.strptime("{}".format(RESERVATION_LAST_DAY), '%Y-%m-%d')

# loop through all the days between first and last day, inclusive
current_day = first_day
while current_day <= last_day:
    current_day_of_the_week = days_of_the_week[current_day.weekday()]
    if current_day_of_the_week in repeat_days:
        start_time = datetime.strptime('{} {}'.format(current_day.strftime('%Y-%m-%d'), RESERVATION_START_TIME),
                                       '%Y-%m-%d %H:%M')
        end_time = datetime.strptime('{} {}'.format(current_day.strftime('%Y-%m-%d'), RESERVATION_END_TIME),
                                     '%Y-%m-%d %H:%M')
        try:
            client.reservation_make(type=RESERVATION_TYPE, pod_id=POD_ID, cls_id=CLASS_ID, start_time=start_time,
                                    end_time=end_time, acc_id=acc_id, tz_olson=RESERVATION_TIME_ZONE)
        except NetlabError as err:
            # The server returned an error. The exception will contain more information about the error.
            reservations_with_errors += 1
            print("  -! {}".format(err))
        else:
            print("  -> Creating reservation at {}".format(start_time))
            reservations_created += 1
    current_day += timedelta(days=1)

print("\nSuccessfully created {} new reservations.".format(reservations_created))
if reservations_with_errors > 0:
    print("There were {} errors encountered. Please review the output above.".format(reservations_with_errors))



# flake8: noqa
from .common import TimeFormat, DateFormat, HDRSeverity, HDRStatus

from .klass import ClassEmailLogs, ClassLabLimit
from .pod import PCIcon, PCType, PLIcon, PodCategory, PodState, RemoveVMS, VirtualHostComPath, VirtualMachineRole, PodAdminState, PodCurrentState
from .reservation import ReservationType, ReservationScope, InitConfig
from .user import AccountPrivileges, AccountType
from .vm import PowerState



from enum import Enum


class PodCategory(Enum):
    """
    Pod category type.
    """
    REAL_EQUIPMENT = 'RE'
    "Real Equipment Pod (i.e. Cisco Equipment)"
    PERSISTENT_VM = 'PV'
    "Persistent VM Pod"
    EPHEMERAL_VM = 'EV'
    "Ephemeral VM Pod"
    MASTER_VM = 'MV'
    "Master VM Pod"
    NORMAL_VM = 'NV'
    "Normal VM Pod"


class PCIcon(Enum):
    """
    Icon Identifier
    """
    PC = "P"
    "PC"
    SERVER = "S"
    "Server"


class PCType(Enum):
    """
    PC Type.
    """
    ABSENT = "ABSENT"
    "PC is absent."
    AVMI = "AVMI"
    "PC is in virtual machine inventory."


class PLIcon(Enum):
    """
    Pod design PC icon identifier.
    """
    PC = "P"
    "PC"
    SERVER = "S"
    "Server"


class VirtualHostComPath(Enum):
    """
    Virtual host communication path.
    """
    OUTSIDE = 'OUTSIDE'
    "Communication will be done on the OUTSIDE network."
    INSIDE = 'INSIDE'
    "Communication will be done on the INSIDE (169.254.0.x) network."


class VirtualMachineRole(Enum):
    """
    Virtual machine role in NETLAB+ inventory.
    """
    NORMAL = 'NORMAL'
    "Virtual machine will typically revert to snapshot after reservation."
    MASTER = 'MASTER'
    """
    Virtual machine is often associated with the Master Pod. Additional protections are in place
    to prevent accidental removal.
    """
    PERSISTENT = 'PERSISTENT'
    """
    Virtual machine will not revert to snapshot after reservation.
    """


class PodState(Enum):
    """
    PC Pod State
    """
    ONLINE = 'ONLINE'
    "Online."
    OFFLINE = 'OFFLINE'
    "Offline"
    RESUME = 'RESUME'
    "Resume"


class RemoveVMS(Enum):
    """

    """
    NONE = 'NONE'
    "None"
    LOCAL = 'LOCAL'
    "Local"
    DATACENTER = 'DATACENTER'
    "Datacenter"
    DISK = 'DISK'
    "Disk"


class PodAdminState(Enum):
    """
    This enumeration describes the administrative states of a pod.

    The administrative state is the state that is desired, not necessarily the current running
    state (:py:class:`netlab.enums.PodAdminState`).
    """
    OFFLINE = "OFFLINE"
    "Pod is offline and idle."
    ONLINE = "ONLINE"
    "Pod is online and idle."
    ACTIVE_INIT = "ACTIVE_INIT"
    "Pod is active and initializing."
    ACTIVE_LOAD = "ACTIVE_LOAD"
    "Pod is active and loading configuration."
    ACTIVE_LAB = "ACTIVE_LAB"
    "Pod is active and available for user interaction."
    ACTIVE_POST = "ACTIVE_POST"
    "Pod is active but no longer available for interaction."
    ACTIVE_SAVE = "ACTIVE_SAVE"
    "Pod is active and saving state."
    ACTIVE_TERM = "ACTIVE_TERM"
    "Pod is active and cleaning up."
    CLONING = "CLONING"
    "Pod is in the process of being cloned."
    SUSPENDED = "SUSPENDED"
    "Pod is suspended due to errors."
    RESUME = "RESUME"
    "Pod is resuming from a suspended state."


class PodCurrentState(Enum):
    """
    This enumeration describes the possible running states of a pod.
    """
    OFFLINE = "OFFLINE"
    "Pod is offline and idle."
    ONLINE = "ONLINE"
    "Pod is online and idle."
    ACTIVE_INIT = "ACTIVE_INIT"
    "Pod is active and initializing."
    ACTIVE_LOAD = "ACTIVE_LOAD"
    "Pod is active and loading configuration."
    ACTIVE_LAB = "ACTIVE_LAB"
    "Pod is active and available for user interaction."
    ACTIVE_POST = "ACTIVE_POST"
    "Pod is active but no longer available for interaction."
    ACTIVE_SAVE = "ACTIVE_SAVE"
    "Pod is active and saving state."
    ACTIVE_TERM = "ACTIVE_TERM"
    "Pod is active and cleaning up."
    CLONING = "CLONING"
    "Pod is in the process of being cloned."
    SUSPENDED = "SUSPENDED"
    "Pod is suspended due to errors."
    RESUME = "RESUME"
    "Pod is resuming from a suspended state."
